#include "../core.h"
#include "../classes/events.h"

#include "../modules/window.h"

void Window::update() {
	draw(false);
}

void Window::draw() {
	draw(false);
}

void Window::draw(bool flagUpdate) {
	if (flagUpdate) kos_WindowRedrawStatus(1);

	kos_DefineAndDrawWindow(x,y,sizeX,sizeY, mainAreaType,mainAreaColour,headerType,headerColour,(Dword)header);

	int i = 0;

	for (i=0;i<MAX_FORM_CONTROLS;i++) {
		if (controls[i]) {
			controls[i]->draw();
			controls[i]->onDraw();
		}
	}

	this->call(EVENT_DRAW);

	if (flagUpdate) kos_WindowRedrawStatus(2);
}

Window::Window() {
	x = 10;
	y = 10;
	sizeX = 500;
	sizeY = 500;
	mainAreaType = 0x33;
	mainAreaColour = COLOR_WHITE;
	headerType = 0;
	headerColour = COLOR_RED;
	header = (Dword)"Window";

	lastControlId = 0;
}

Window::~Window() {
	int i;
	for (i=0;i<MAX_FORM_CONTROLS;i++) {
		delete controls[i];
	}
}

void Window::waitForEvent() {
	int k_event, i;

	while(k_event=kos_WaitForEvent()) {
		for (i=0;i<MAX_FORM_CONTROLS;i++) {
			if (controls[i]) controls[i]->sysevent(k_event);
		}

		switch (k_event) {
			case EM_WINDOW_REDRAW:
				draw();
			break;
			case EM_KEY_PRESS:
				Byte keyCode;
				kos_GetKey(keyCode);
				lastKeyPressCode = keyCode;

				this->call(EVENT_KEY_PRESS);
			break;
			case 3:
				Dword buttonId;
				kos_GetButtonID(buttonId);
				
				clickButtonId = (int)buttonId;

				Button* button = getButton(buttonId);
				if (button) {
					button->call(EVENT_BUTTON_CLICK);
					button->onClick();
				}

				if (buttonId == 1) {
					this->call(EVENT_WINDOW_CLOSE);
					return Application::exit();
				}

			break;
		}
	}
}

Button* Window::getButton(int id) {
	int i;
	for (i=0;i<MAX_FORM_CONTROLS;i++) {
		if (controls[i]->getType() == CONTROL_BUTTON && ((Button*)controls[i])->id == id) return (Button*)controls[i];
	}

	return 0;
}

Control* Window::getControl(int controlType, int controlId) {
	int i;
	for (i=0;i<MAX_FORM_CONTROLS;i++) {
		if (controls[i]->getType() == controlType && controls[i]->getId() == controlId) return controls[i];
		//return controls[i];
		//if (controls[i]->getType() == controlType) return controls[i];
	}

	return 0;
}


int Window::addControl(int type, Control* control) {
	controls[lastControlId] = control;
	control->init();
	lastControlId++;
	
	return (lastControlId-1);
}

int Window::addControl(Control* control) {
	controls[lastControlId] = control;
	control->init();
	lastControlId++;
	
	return (lastControlId-1);
}